from typing import List

from tabulate import tabulate


class TextMetrics:
    def __init__(
        self, Bleu_1=0, Bleu_2=0, Bleu_3=0, Bleu_4=0, METEOR=0, ROUGE_L=0, CIDEr=0
    ):
        self.bleu_1 = Bleu_1
        self.bleu_2 = Bleu_2
        self.bleu_3 = Bleu_3
        self.bleu_4 = Bleu_4
        self.meteor = METEOR
        self.rouge_l = ROUGE_L
        self.cider = CIDEr

    def __add__(self, other):
        if type(other) is TextMetrics:
            return TextMetrics(
                self.bleu_1 + other.bleu_1,
                self.bleu_2 + other.bleu_2,
                self.bleu_3 + other.bleu_3,
                self.bleu_4 + other.bleu_4,
                self.meteor + other.meteor,
                self.rouge_l + other.rouge_l,
                self.cider + other.cider,
            )
        elif type(other) is int:
            return TextMetrics(
                self.bleu_1 + other,
                self.bleu_2 + other,
                self.bleu_3 + other,
                self.bleu_4 + other,
                self.meteor + other,
                self.rouge_l + other,
                self.cider + other,
            )
        else:
            return NotImplementedError

    def __truediv__(self, other):
        return TextMetrics(
            self.bleu_1 / other,
            self.bleu_2 / other,
            self.bleu_3 / other,
            self.bleu_4 / other,
            self.meteor / other,
            self.rouge_l / other,
            self.cider / other,
        )

    def __sub__(self, other):
        if type(other) is TextMetrics:
            return TextMetrics(
                self.bleu_1 - other.bleu_1,
                self.bleu_2 - other.bleu_2,
                self.bleu_3 - other.bleu_3,
                self.bleu_4 - other.bleu_4,
                self.meteor - other.meteor,
                self.rouge_l - other.rouge_l,
                self.cider - other.cider,
            )
        else:
            return NotImplementedError

    def __pow__(self, other):
        return TextMetrics(
            self.bleu_1**other,
            self.bleu_2**other,
            self.bleu_3**other,
            self.bleu_4**other,
            self.meteor**other,
            self.rouge_l**other,
            self.cider**other,
        )


class Scores:
    def __init__(
        self,
        text_metrics_it=None,
        text_metrics_ti=None,
    ):
        if text_metrics_it is None:
            text_metrics_it = TextMetrics()
        if text_metrics_ti is None:
            text_metrics_ti = TextMetrics()
        self.text_metrics_it = (
            text_metrics_it
            if type(text_metrics_it) is TextMetrics
            else TextMetrics(**text_metrics_it)
        )
        self.text_metrics_ti = (
            text_metrics_ti
            if type(text_metrics_ti) is TextMetrics
            else TextMetrics(**text_metrics_ti)
        )

    def __add__(self, other):
        if type(other) is int:
            return Scores(
                self.text_metrics_it + other,
                self.text_metrics_ti + other,
            )
        elif type(other) is Scores:
            return Scores(
                self.text_metrics_it + other.text_metrics_it,
                self.text_metrics_ti + other.text_metrics_ti,
            )
        else:
            return NotImplementedError

    def __truediv__(self, other):
        return Scores(
            self.text_metrics_it / other,
            self.text_metrics_ti / other,
        )

    def __sub__(self, other):
        return Scores(
            self.text_metrics_it - other.text_metrics_it,
            self.text_metrics_ti - other.text_metrics_ti,
        )

    def __pow__(self, other):
        return Scores(
            self.text_metrics_it**other,
            self.text_metrics_ti**other,
        )

    def __radd__(self, other):
        if type(other) is int:
            if other == 0:
                return self
            else:
                return NotImplementedError
        elif type(other) is Scores:
            return self + other
        else:
            return NotImplementedError

    def __gt__(self, other):
        if isinstance(other, Scores):
            return self.text_metrics_ti.bleu_4 > other.text_metrics_ti.bleu_4
        else:
            return NotImplementedError

    def __str__(self):
        avg_txt_metrics = self.get_average_scores_rounded()

        table = [
            ["", "I -> T", "T -> I", "Average"],
            [
                "BLEU-1",
                round(self.text_metrics_it.bleu_1, 3),
                round(self.text_metrics_ti.bleu_1, 3),
                avg_txt_metrics.bleu_1,
            ],
            [
                "BLEU-2",
                round(self.text_metrics_it.bleu_2, 3),
                round(self.text_metrics_ti.bleu_2, 3),
                avg_txt_metrics.bleu_2,
            ],
            [
                "BLEU-3",
                round(self.text_metrics_it.bleu_3, 3),
                round(self.text_metrics_ti.bleu_3, 3),
                avg_txt_metrics.bleu_3,
            ],
            [
                "BLEU-4",
                round(self.text_metrics_it.bleu_4, 3),
                round(self.text_metrics_ti.bleu_4, 3),
                avg_txt_metrics.bleu_4,
            ],
            [
                "METEOR",
                round(self.text_metrics_it.meteor, 3),
                round(self.text_metrics_ti.meteor, 3),
                avg_txt_metrics.meteor,
            ],
            [
                "ROUGE-L",
                round(self.text_metrics_it.rouge_l, 3),
                round(self.text_metrics_ti.rouge_l, 3),
                avg_txt_metrics.rouge_l,
            ],
            [
                "CIDEr",
                round(self.text_metrics_it.cider, 3),
                round(self.text_metrics_ti.cider, 3),
                avg_txt_metrics.cider,
            ],
        ]

        return tabulate(table, headers="firstrow", tablefmt="grid")

    def get_average_scores_rounded(self):
        return TextMetrics(
            round((self.text_metrics_it.bleu_1 + self.text_metrics_ti.bleu_1) / 2, 3),
            round((self.text_metrics_it.bleu_2 + self.text_metrics_ti.bleu_2) / 2, 3),
            round((self.text_metrics_it.bleu_3 + self.text_metrics_ti.bleu_3) / 2, 3),
            round((self.text_metrics_it.bleu_4 + self.text_metrics_ti.bleu_4) / 2, 3),
            round((self.text_metrics_it.meteor + self.text_metrics_ti.meteor) / 2, 3),
            round((self.text_metrics_it.rouge_l + self.text_metrics_ti.rouge_l) / 2, 3),
            round((self.text_metrics_it.cider + self.text_metrics_ti.cider) / 2, 3),
        )

    def get_latex(self):
        avg_txt_metrics = self.get_average_scores_rounded()
        return f"{'{:.3f}'.format(round(self.text_metrics_it.bleu_1,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.bleu_1,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.bleu_1)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.bleu_2,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.bleu_2,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.bleu_2)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.bleu_3,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.bleu_3,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.bleu_3)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.bleu_4,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.bleu_4,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.bleu_4)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.meteor,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.meteor,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.meteor)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.rouge_l,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.rouge_l,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.rouge_l)} & \
                 {'{:.3f}'.format(round(self.text_metrics_it.cider,3))} & \
                 {'{:.3f}'.format(round(self.text_metrics_ti.cider,3))} & \
                 {'{:.3f}'.format(avg_txt_metrics.cider)} \\\\[0.1cm]"


class BestScores:
    def __init__(
        self,
        full_set=None,
        change_set=None,
        nochange_set=None,
    ):
        self.full_set = full_set if full_set is not None else Scores()
        self.change_set = change_set if change_set is not None else Scores()
        self.nochange_set = nochange_set if nochange_set is not None else Scores()

    def __str__(self) -> str:
        return (
            f"Full Set:\n {str(self.full_set)}\n"
            f"Change Set:\n {str(self.change_set)}\n"
            f"No Change Set:\n {str(self.nochange_set)}\n"
        )

    def get_latex(self):
        return (
            f"& All & {self.full_set.get_latex()}\n"
            f"& Change & {self.change_set.get_latex()}\n"
            f"& No Change & {self.nochange_set.get_latex()}\n"
        )


class RunningScores:
    def __init__(self, full_set=None, change_set=None, nochange_set=None):
        self.full_set: List[Scores] = [] if full_set is None else full_set
        self.change_set: List[Scores] = [] if change_set is None else change_set
        self.nochange_set: List[Scores] = [] if nochange_set is None else nochange_set

    def __add__(self, other):
        return RunningScores(
            [x + y for x, y in zip(self.full_set, other.full_set)],
            [x + y for x, y in zip(self.change_set, other.change_set)],
            [x + y for x, y in zip(self.nochange_set, other.nochange_set)],
        )

    def __radd__(self, other):
        if type(other) is int:
            if other == 0:
                return self
            else:
                return NotImplementedError
        elif type(other) is RunningScores:
            return self + other
        else:
            return NotImplementedError

    def __truediv__(self, value):
        return RunningScores(
            [scores / value for scores in self.full_set],
            [scores / value for scores in self.change_set],
            [scores / value for scores in self.nochange_set],
        )

    def __sub__(self, other):
        return RunningScores(
            [x - y for x, y in zip(self.full_set, other.full_set)],
            [x - y for x, y in zip(self.change_set, other.change_set)],
            [x - y for x, y in zip(self.nochange_set, other.nochange_set)],
        )

    def __pow__(self, value):
        return RunningScores(
            [scores**value for scores in self.full_set],
            [scores**value for scores in self.change_set],
            [scores**value for scores in self.nochange_set],
        )

    def __str__(self):
        full_set_str = "\n".join([str(s) for s in self.full_set])
        change_set_str = "\n".join([str(s) for s in self.change_set])
        no_change_set_str = "\n".join([str(s) for s in self.nochange_set])

        return f"Full set:\n{full_set_str}\nChange set:\n{change_set_str}\nNo change set:\n{no_change_set_str}\n"
